@@ -1,4 +1,4 @@
-"bioenv.formula" <-
+`bioenv.formula` <-
     function (formula, data, ...) 
 {
     if (missing(data)) 
@@ -7,12 +7,7 @@
     comm <- formula[[2]]
     comm <- eval(comm, data, parent.frame())
     formula[[2]] <- NULL
-    mf <- model.frame(formula, data, na.action = na.fail)
-    if (any(sapply(mf, function(x) is.factor(x) || !is.numeric(x)))) 
-        stop("bioenv applies only to numeric variables")
-    env <- attr(mf, "terms")
-    attr(env, "intercept") <- 0
-    env <- model.matrix(env, mf)
+    env <- model.frame(formula, data, na.action = NULL)
     out <- bioenv(comm, env, ...)
     out$formula <- fla
     out$call <- match.call()
