@@ -8,8 +8,10 @@
     type <- match.arg(type)
     ## Return scaled eigenvalues
     U <- switch(model,
-                CCA = object$CCA$u %*% diag(sqrt(object$CCA$eig)),
-                CA = object$CA$u %*% diag(sqrt(object$CA$eig)),
+                CCA = object$CCA$u %*% diag(sqrt(object$CCA$eig),
+                                            nrow = length(object$CCA$eig)),
+                CA = object$CA$u %*% diag(sqrt(object$CA$eig),
+                                          nrow = length(object$CA$eig)),
                 Imaginary = object$CA$imaginary.u.eig,
                 pCCA = object$pCCA$Fit/object$adjust)
     if (is.null(U))
