@@ -1,10 +1,10 @@
 "summary.humpfit" <-
     function (object, ...) 
 {
-    dispersion <- if (any(object$family$family == c("binomial", "poisson")))
+    dispersion <- if (any(object$family$family == c("binomial", 
+                          "poisson"))) 
         1
-    else
-        sum(object$residuals^2)/object$df.residual
+    else sum(object$residuals^2)/object$df.residual
     p <- coef(object)
     se <- sqrt(dispersion * diag(solve(object$nlm$hessian)))
     est <- cbind(p, se)
@@ -17,9 +17,10 @@
     aic <- AIC(object)
     bic <- AIC(object, k = log(length(object$y)))
     out <- list(est = est, aic = aic, bic = bic, family = family(object)$family, 
-                deviance = deviance(object), df.residual = df.residual(object),
-                dispersion = dispersion,
-                correlation = cormat, iter = object$nlm$iterations, code = object$nlm$code)
+                deviance = deviance(object), df.residual = df.residual(object), 
+                dispersion = dispersion, correlation = cormat,
+                cov.unscaled = covmat,  iter = object$nlm$iterations,
+                code = object$nlm$code)
     class(out) <- "summary.humpfit"
     out
 }
