@@ -600,9 +600,9 @@ C OTHERWISE, UPDATE STEP SIZE.
 C
       ELSE
         FACTR1=4.0**COSAV
-        FACTR2=1.6/( (1.0+(MIN(1.0,SRATAV))**5) *
+        FACTR2=1.6/( (1.0+(MIN(1D0,SRATAV))**5) *
      .    (1.0+ACOSAV-ABS(COSAV)) )
-        FACTR3=SQRT(MIN(1.0,SRATIO))
+        FACTR3=SQRT(MIN(1D0,SRATIO))
         STEP=STEP*FACTR1*FACTR2*FACTR3
       ENDIF
       RETURN
